"{ Package: 'stx:libbasic2' }"

Object subclass:#Archiver
	instanceVariableNames:'process temporaryDirectory fileName outStream errorStream
		synchron'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Support-FileFormats'
!

Archiver subclass:#GZipArchive
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Archiver
!

Archiver subclass:#MultiFileArchive
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Archiver
!

Archiver::MultiFileArchive subclass:#TarArchive
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Archiver
!

Archiver subclass:#TarGZipArchive
	instanceVariableNames:'tarArchiver tarFile'
	classVariableNames:''
	poolDictionaries:''
	privateIn:Archiver
!

Archiver::MultiFileArchive subclass:#ZipArchive
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:Archiver
!


!Archiver class methodsFor:'instance creation'!

classForMimeType:aMimeType
    |className|

    aMimeType isNil ifTrue:[^ nil].

    className := (
            #(
                ('application/x-tar-compressed'      TarGZipArchive     )
                ('application/x-tar'                 TarArchive         )
                ('application/x-gzip-compressed'     GZipArchive        )
                ('application/x-zip-compressed'      ZipArchive         )
                ('application/x-squeak-archive'      ZipArchive         )
                ('application/java-archive'          ZipArchive         )
            ) detect:[:entry | entry first = aMimeType] ifNone:#(nil nil)
        ) last.
    className isNil ifTrue:[^ nil].
    ^ self privateClassesAt:className.

    "
     self classForMimeType:'application/x-tar'    
     self classForMimeType:'application/x-foo'    
     self classForMimeType:'application/x-squeak-archive'   
     self classForMimeType:'application/java-archive'   
     'foo.sar' asFilename mimeTypeFromName               
    "
!

newFor:aFilename
    |mimeType archiverClass|

    mimeType := aFilename asFilename mimeTypeFromName.
    archiverClass := self classForMimeType:mimeType.
    archiverClass isNil ifTrue:[^ nil].
    ^ archiverClass with:aFilename
!

with:aFilename

    | instance |

    instance := self new.
    instance fileName:aFilename.
    ^ instance
! !

!Archiver class methodsFor:'classAccess'!

gzipArchive

    ^ GZipArchive
!

tarArchive

    ^ TarArchive
!

tarGZipArchive

    ^ TarGZipArchive
!

zipArchive

    ^ ZipArchive
! !

!Archiver class methodsFor:'queries'!

canAddFiles

    ^ false
!

canDragnDrop

    ^ true.
!

canExtractFiles

    ^ false
!

canRemoveFiles

    ^ false
!

canViewFile

    ^ false
!

hasTitleLine

    ^ false
! !

!Archiver methodsFor:'accessing'!

errorStream
    "return the value of the instance variable 'errorStream' (automatically generated)"

    ^ errorStream
!

errorStream:something
    "set the value of the instance variable 'errorStream' (automatically generated)"

    errorStream := something.
!

fileName
    "return the value of the instance variable 'fileName' (automatically generated)"

    ^ fileName
!

fileName:something
    "set the value of the instance variable 'fileName' (automatically generated)"

    fileName := something.
!

outStream
    "return the value of the instance variable 'outStream' (automatically generated)"

    ^ outStream
!

outStream:something
    "set the value of the instance variable 'outStream' (automatically generated)"

    outStream := something.
!

process
    "return the value of the instance variable 'process' (automatically generated)"

    ^ process
!

process:something
    "set the value of the instance variable 'process' (automatically generated)"

    process := something.
!

synchron
    "return the value of the instance variable 'synchron' (automatically generated)"

    ^ synchron
!

synchron:something
    "set the value of the instance variable 'synchron' (automatically generated)"

    synchron := something.
!

temporaryDirectory
    "return the value of the instance variable 'temporaryDirectory' (automatically generated)"

    temporaryDirectory isNil ifTrue:[
        temporaryDirectory := Filename newTemporary.
        temporaryDirectory makeDirectory.
    ].
    ^ temporaryDirectory
! !

!Archiver methodsFor:'actions'!

extractFilesTo:aDirectory
    self extractFiles:nil to:aDirectory
!

removeFilesFromArchive:aColOfFiles

    self subclassResponsibility.
! !

!Archiver methodsFor:'actions-basic'!

addFilesToArchive:colOfFiles
    self subclassResponsibility.
!

extractFiles:aColOfFilesOrNil to:aDirectory
    self subclassResponsibility.
!

listFiles:aColOfFiles
    self subclassResponsibility.
! !

!Archiver methodsFor:'actions-private'!

removeTemporaryDirectory

    | tmp |

    temporaryDirectory notNil ifTrue:[
        tmp := self temporaryDirectory.
        (FileDirectory directoryNamed:(tmp directory)) removeDirectory:tmp baseName.
        temporaryDirectory := nil.
    ].
!

stopProcess

    process notNil ifTrue:[
        process terminateWithAllSubprocesses.
        process waitUntilTerminated.
    ].
! !

!Archiver methodsFor:'command execution'!

executeCommand:cmd directory:aDirectory


    synchron isNil ifTrue:[synchron := true].
    synchron ifTrue:[
         OperatingSystem 
            executeCommand:cmd
            inputFrom:nil
            outputTo:outStream
            errorTo:errorStream
            inDirectory:aDirectory
            lineWise:true
            onError:[:status| false].
    ] ifFalse:[
        process := Process for:[
                [ 
                     OperatingSystem 
                        executeCommand:cmd
                        inputFrom:nil
                        outputTo:outStream
                        errorTo:errorStream
                        inDirectory:aDirectory
                        lineWise:true
                        onError:[:status| false].
                ] ensure:[
                    process := nil.
                ].

        ] priority:(Processor systemBackgroundPriority).
        process name:('Archiver command: ', cmd).
        process resume.
    ]
!

outStream:aOutStream errorStream:aErrorStream

    outStream := aOutStream.
    errorStream := aErrorStream.
!

outStream:aOutStream errorStream:aErrorStream synchron:aBoolean

    outStream := aOutStream.
    errorStream := aErrorStream.
    synchron := aBoolean
! !

!Archiver methodsFor:'initialization & release'!

release

    self stopProcess.
    self removeTemporaryDirectory.
! !

!Archiver::GZipArchive class methodsFor:'columns'!

columns

    "/ columns in stream order
    "/  colums id          words to read
    ^ #( #(#method          1)      
         #(#crc             1)    
         #(#dateAndTime     3)   
         #(#compressSize    1)  
         #(#size            1) 
         #(#ratio           1) 
         #(#fileName        #rest)
    ) 
! !

!Archiver::GZipArchive class methodsFor:'command strings'!

gunzipCommand
    ^ 'gunzip'
!

gzipCommand
    ^ 'gzip'
! !

!Archiver::GZipArchive class methodsFor:'queries'!

canViewFile

    ^ true
!

hasTitleLine

    ^ true
! !

!Archiver::GZipArchive methodsFor:'actions'!

extractFiles:aColOfFilesOrNil to:aDirectory
    self unzipTo:aDirectory
!

listFiles:aColOfFilesOrNil
    self fileName isNil ifTrue:[ ^ self].

    outStream nextPutLine:self fileName withoutSuffix pathName.

"/    ^ Array with: (self fileName withoutSuffix)
!

unzipTo:aDirectory
    | cmd file newFile|

    (aDirectory exists not) ifTrue:[
        DialogBox warn:'cant unzip to not existing directory ', aDirectory asString.
    ].
    (aDirectory isDirectory not) ifTrue:[
        DialogBox warn:'cant unzip to file ', aDirectory asString.
    ].
    file := newFile := self fileName.
    (file directory pathName = aDirectory pathName) ifFalse:[
        newFile := aDirectory construct:(file baseName).
        file copyTo:newFile.
    ].
    cmd := self getCommandToGunzip:newFile.
    self executeCommand:cmd directory:aDirectory. 
!

zipFile:aFile to:newFile
    | cmd directory|

    directory := newFile directory.
    (directory exists) ifFalse:[
        DialogBox warn:'cannot gzip to non-existing directory ', directory asString.
    ].
    (directory isDirectory) ifFalse:[
        DialogBox warn:'cannot gzip to non-directory ', directory asString.
    ].

    cmd := self getCommandToGZip:aFile asNew:newFile.
    self executeCommand:cmd directory:directory.
    newFile exists ifTrue:[
        self fileName:newFile.
    ].
! !

!Archiver::GZipArchive methodsFor:'command strings'!

getCommandToGZip:aFile asNew:newFile
    ^ '%1 -c %2 > %3' 
        bindWith:self class gzipCommand
        with:aFile asString 
        with:newFile asString
!

getCommandToGunzip:aFileName 
    ^ '%1 %2' 
        bindWith:self class gunzipCommand
        with:aFileName baseName
! !

!Archiver::MultiFileArchive methodsFor:'actions'!

addFilesToArchive:colOfFiles
    |cmd tempDir archivFile archivInTemp|

    archivFile := self fileName.

    tempDir := self temporaryDirectory.
    archivInTemp := tempDir construct:(archivFile baseName).
    "/ copy files to be added to tempDir
    colOfFiles do:[:file |
        file recursiveCopyTo:(tempDir construct:(file asFilename baseName))
    ].

    "/ copy archiv to tempDir
    archivFile copyTo:archivInTemp.

    "/ addFiles to the tar archive
    cmd := self getCommandToAdd:colOfFiles toArchive:archivInTemp.
    self executeCommand:cmd directory:tempDir.

    "/ copy tar archiv back
    archivInTemp copyTo:(self fileName).

    "/ cg: remove the tempFile
    archivInTemp remove.
!

extractFiles:aColOfFilesOrNil to:aDirectory
    |execDir cmd|

    execDir := self fileName directory.
    cmd := self getCommandToExtractFiles:aColOfFilesOrNil intoDirectory:aDirectory.
    self executeCommand:cmd directory:execDir.
!

extractFiles:aColOfFiles withoutDirectoryTo:aDirectory
    |execDir tempDir tempFile targetFile|

    execDir := self fileName directory.
    tempDir := self temporaryDirectory.
    self extractFiles:aColOfFiles to:tempDir.

    aColOfFiles do:[ : aFileString |
        tempFile := self temporaryDirectory construct:aFileString.
        targetFile := aDirectory construct:(aFileString asFilename baseName).
        targetFile exists ifTrue:[
            targetFile recursiveRemove.
        ].
        tempFile exists ifTrue:[
            tempFile recursiveCopyTo:targetFile.
        ].
    ].
!

listFiles:aColOfFiles
    |cmd dir|

    self fileName isNil ifTrue:[ ^ self].
    dir := self fileName directory.
    cmd := self getCommandToListFiles:aColOfFiles.
    self executeCommand:cmd directory:dir 
!

removeFilesFromArchive:aColOfFiles

    |cmd|

    cmd := self getCommandToRemoveFiles:aColOfFiles.
    self executeCommand:cmd directory:(self fileName directory). 
! !

!Archiver::MultiFileArchive methodsFor:'command strings'!

getCommandToAdd:colOfFiles toArchive:archivIn
    self subclassResponsibility
!

getCommandToExtractFiles:sel intoDirectory:dir
    self subclassResponsibility
!

getCommandToListFiles:aColOfFiles
    self subclassResponsibility
!

getCommandToRemoveFiles:aColOfFiles
    self subclassResponsibility
! !

!Archiver::TarArchive class methodsFor:'columns'!

columns

    "/  colums id          words to read
    ^ #( #(#permissions     1)      
         #(#ownerGroup      1)    
         #(#size            1)   
         #(#dateAndTime     2)  
         #(#fileName        #rest)
    ) 
! !

!Archiver::TarArchive class methodsFor:'command strings'!

stringWithQuotedFileBaseNames:aColOfFiles

    aColOfFiles isNil ifTrue:[^ ''].

    ^ String 
        streamContents:[:str |
            aColOfFiles do:[:fn | 
                str nextPutAll:' "'.
                str nextPutAll:(fn asFilename baseName).
                str nextPutAll:'"'.
            ].
        ]
!

tarCommand
    ^ 'tar'
! !

!Archiver::TarArchive class methodsFor:'queries'!

canAddFiles

    ^ true
!

canRemoveFiles

    ^ true
!

canViewFile

    ^ true
! !

!Archiver::TarArchive methodsFor:'command strings'!

getCommandToAdd:aColOfFiles toArchive:archiveFile
    |stream|

    archiveFile exists ifFalse:[^ nil].

    stream := WriteStream on:''.

    "/ 'r'  TarArchivAddOption
    "/ 'f'  TarArchivFileOption
    stream nextPutAll:('%1 -rf "%2"' 
                    bindWith:self class tarCommand
                    with:archiveFile asString string).

    stream nextPutAll:(self class stringWithQuotedFileBaseNames:aColOfFiles).
    ^ stream contents
!

getCommandToExtractFiles:sel intoDirectory:dir 
    |stream|

    stream := WriteStream on:''.

    "/ 'x'  TarArchivUnpackOption
    "/ 'f'  TarArchivFileOption
    "/ 'C'  TarArchivUnpackInDirectoryOption
    stream nextPutAll:('%1 -xf "%2" -C %3' 
                    bindWith:self class tarCommand
                    with:self fileName asString string
                    with:dir asString).

    sel notNil ifTrue:[
        sel do:[:el | 
            stream nextPutAll:' "'.
            stream nextPutAll:(el asString).
            stream nextPutAll:'"'
        ].
    ].
    ^ stream contents.
!

getCommandToListFiles:aColOfFiles 
    |stream|

    stream := WriteStream on:''.

    "/ 't'  TarArchivListContentsOption
    "/ 'v'  TarArchivVerboseOption
    "/ 'f'  TarArchivFileOption
    stream nextPutAll:('%1 -tvf "%2"' 
                    bindWith:self class tarCommand
                    with:self fileName baseName).

    stream nextPutAll:(self class stringWithQuotedFileBaseNames:aColOfFiles).
    ^ stream contents.
!

getCommandToRemoveFiles:sel 
    |stream filename|

    filename := self fileName.
    filename exists ifFalse:[^ nil].

    stream := WriteStream on:''.

    "/ 'f'  TarArchivFileOption
    stream nextPutAll:('%1 --delete -f "%2"' 
                    bindWith:self class tarCommand
                    with:self fileName baseName).

    stream nextPutAll:(self class stringWithQuotedFileBaseNames:sel).
    ^ stream contents
! !

!Archiver::TarGZipArchive class methodsFor:'columns'!

columns

    ^ Archiver tarArchive columns
! !

!Archiver::TarGZipArchive class methodsFor:'queries'!

canAddFiles

    ^ Archiver tarArchive canAddFiles
!

canRemoveFiles

    ^ Archiver tarArchive canRemoveFiles
!

canViewFile

    ^ Archiver tarArchive canViewFile
! !

!Archiver::TarGZipArchive methodsFor:'accessing'!

fileName:aFile

    | tempDir file gzipArchiver suffix|

    super fileName:aFile.
    " unzip file in tempDirectory and do all the things with tar file "
    tempDir := self temporaryDirectory.
    gzipArchiver := Archiver::GZipArchive with:(self fileName).
    gzipArchiver unzipTo:tempDir.
    suffix := self fileName suffix.
    file := self fileName withoutSuffix.
    file := file baseName.
    suffix = 'tgz' ifTrue:[
        file := file , '.tar'
    ].
    tarFile := self temporaryDirectory construct:file.
    tarArchiver := Archiver::TarArchive with:tarFile.
! !

!Archiver::TarGZipArchive methodsFor:'actions'!

addFilesToArchive:colOfFiles

    self setCommandOptions.
    tarArchiver addFilesToArchive:colOfFiles.
    "/ synchronize the tar archive under temporary file with archiv file position
    self synchronize.
!

extractFiles:aColOfFiles to:aDirectory

    self setCommandOptions.
    tarArchiver extractFiles:aColOfFiles to:aDirectory.
!

extractFiles:aColOfFiles withOutDirectoryTo:aDirectory

    self setCommandOptions.
    tarArchiver extractFiles:aColOfFiles withOutDirectoryTo:aDirectory.
!

listFiles:aColOfFiles
    self setCommandOptions.
    tarArchiver listFiles:aColOfFiles.
!

removeFilesFromArchive:aColOfFiles

    self setCommandOptions.
    tarArchiver removeFilesFromArchive:aColOfFiles.
    self synchronize.
! !

!Archiver::TarGZipArchive methodsFor:'actions private'!

setCommandOptions

    tarArchiver outStream:(self outStream).
    tarArchiver errorStream:(self errorStream).
    tarArchiver synchron:(self synchron).
!

synchronize

    |gzipArchiver|

    gzipArchiver := Archiver::GZipArchive with:nil.
    gzipArchiver zipFile:(tarArchiver fileName) to:(self fileName).
! !

!Archiver::TarGZipArchive methodsFor:'initialization & release'!

release

    super release.
    tarArchiver release.
! !

!Archiver::ZipArchive class methodsFor:'columns'!

columns

    "/  colums id          words to read
    ^ #( (#permissions     1)      
         (#version         2)    
         (#size            1)     
         (#type            1)    
         (#ratio           1)    
         (nil              1)  
         (#dateAndTime     2)  
         (#fileName        #rest)
    ) 
! !

!Archiver::ZipArchive class methodsFor:'command strings'!

unzipCommand
    ^ 'unzip'
!

zipCommand
    ^ 'zip'
! !

!Archiver::ZipArchive class methodsFor:'queries'!

canAddFiles

    ^ true
!

canRemoveFiles

    ^ true
!

canViewFile

    ^ true
!

hasLastLine
    ^ true
!

hasTitleLine
    ^ true
! !

!Archiver::ZipArchive methodsFor:'command strings'!

getCommandToAdd:aColOfFiles toArchive:archiveFile
    |stream|

    archiveFile exists ifFalse:[^ nil].

    stream := WriteStream on:''.
    
    stream nextPutAll:('%1 -r "%2"' 
                    bindWith:self class zipCommand
                    with:archiveFile asString string).

    aColOfFiles do:[:el | 
        stream nextPutAll:' "'.
        stream nextPutAll:(el asFilename baseName).
        stream nextPutAll:'"'
    ].

    ^ stream contents
!

getCommandToExtractFiles:sel intoDirectory:dir
    |stream|

    stream := WriteStream on:''.

    "/ -o   UnzipOverwriteExistingFilesOption
    "/ -d   UnzipExtDirectoryOption

    stream nextPutAll:('%1 -o -d "%2" "%3"' 
                    bindWith:self class unzipCommand
                    with:dir asString string
                    with:self fileName asString).

    sel notNil ifTrue:[
        sel do:[:el | 
            stream nextPutAll:' "'.
            stream nextPutAll:(el asString).
            stream nextPutAll:'"'
        ].
    ].
    ^ stream contents.
!

getCommandToListFiles:aColOfFiles 
    |stream|

    stream := WriteStream on:''.

    "/  -Z      ZipInfoOption
    "/  -h     ZipHeaderOption
    "/  -t      ZipTotalOption
    stream nextPutAll:('%1 -Z -m -h "%2"' 
                        bindWith:self class unzipCommand
                        with:self fileName asString string).

    aColOfFiles notNil ifTrue:[
        aColOfFiles do:[:el | 
            stream nextPutAll:' "'.
            stream nextPutAll:(el baseName).
            stream nextPutAll:'"'
        ]
    ].
    ^ stream contents.
!

getCommandToRemoveFiles:aColOfFiles 
    |stream|

    stream := WriteStream on:''.

    stream nextPutAll:('%1 -d "%2"' 
                        bindWith:self class zipCommand
                        with:self fileName asString string).

    aColOfFiles do:[:el | 
        stream nextPutAll:' "'.
        stream nextPutAll:(el asString).
        stream nextPutAll:'"'
    ].
    ^ stream contents.
! !

!Archiver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic2/Archiver.st,v 1.16 2003-05-16 18:16:36 cg Exp $'
! !
